@@ -43,6 +43,7 @@ from chex._src.asserts import assert_tree_all_equal_shapes
 from chex._src.asserts import assert_tree_all_equal_structs
 from chex._src.asserts import assert_tree_all_finite
 from chex._src.asserts import assert_type
+from chex._src.asserts import clear_trace_counter
 from chex._src.asserts import if_args_not_none
 from chex._src.dataclass import dataclass
 from chex._src.dataclass import mappable_dataclass
@@ -67,7 +68,7 @@ from chex._src.variants import TestCase
 from chex._src.variants import variants
 
 
-__version__ = "0.0.3"
+__version__ = "0.0.4"
 
 __all__ = (
     "all_variants",
@@ -106,6 +107,7 @@ __all__ = (
     "assert_tree_all_finite",
     "assert_type",
     "ChexVariantType",
+    "clear_trace_counter",
     "dataclass",
     "fake_jit",
     "fake_pmap",
@@ -133,7 +135,4 @@ __all__ = (
 #                 ||----w |
 #                 ||     ||
 #
-try:
-  del _src  # pylint: disable=undefined-variable
-except NameError:
-  pass
+
