@@ -302,7 +302,50 @@ class DataclassesTest(parameterized.TestCase):
   )
   def test_tuple_rev_conversion(self, frozen):
     obj = dummy_dataclass(frozen=frozen)
-    asserts.assert_tree_all_close(obj.__class__.from_tuple(obj.to_tuple()), obj)
+    asserts.assert_tree_all_close(type(obj).from_tuple(obj.to_tuple()), obj)
+
+  @parameterized.named_parameters(
+      ('frozen', True),
+      ('mutable', False),
+  )
+  def test_inheritance(self, frozen):
+
+    @chex_dataclass(frozen=frozen)
+    class Base:
+      x: int
+
+    @chex_dataclass(frozen=frozen)
+    class Derived(Base):
+      y: int
+
+    base_obj = Base(x=1)
+    self.assertNotIsInstance(base_obj, Derived)
+    self.assertIsInstance(base_obj, Base)
+
+    derived_obj = Derived(x=1, y=2)
+    self.assertIsInstance(derived_obj, Derived)
+    self.assertIsInstance(derived_obj, Base)
+
+  def test_inheritance_from_empty_frozen_base(self):
+
+    @chex_dataclass(frozen=True)
+    class FrozenBase:
+      pass
+
+    @chex_dataclass(frozen=True)
+    class DerivedFrozen(FrozenBase):
+      j: int
+
+    df = DerivedFrozen(j=2)
+    self.assertIsInstance(df, FrozenBase)
+
+    with self.assertRaisesRegex(
+        TypeError, 'cannot inherit non-frozen dataclass from a frozen one'):
+
+      @chex_dataclass  # pylint:disable=unused-variable
+      class DerivedMutable(FrozenBase):
+        j: int
+
 
 if __name__ == '__main__':
   absltest.main()
