@@ -66,7 +66,7 @@ def set_n_cpu_devices(n: Optional[int] = None):
   Raises:
     RuntimeError: if XLA backends were already initialized.
   """
-  n = n or FLAGS.chex_n_cpu_devices
+  n = n or FLAGS['chex_n_cpu_devices'].value
 
   n_devices = get_n_cpu_devices_from_xla_flags()
   cpu_backend = (jax.lib.xla_client._local_backends or {}).get('cpu', None)  # pylint: disable=protected-access
