@@ -48,10 +48,9 @@ DEFAULT_NAMED_PARAMS = (('case_0', 1, 2, 1), ('case_1', 4, 6, 2))
 
 # Set `FLAGS.chex_n_cpu_devices` CPU devices for all tests.
 def setUpModule():
-  # Has the same effect as `fake.set_n_cpu_devices(FLAGS.chex_n_cpu_devices)`.
   fake.set_n_cpu_devices()
   asserts.assert_devices_available(
-      FLAGS.chex_n_cpu_devices, 'cpu', backend='cpu')
+      FLAGS['chex_n_cpu_devices'].value, 'cpu', backend='cpu')
 
 
 def _scalar_to_ndarray(x, shape=None):
@@ -719,7 +718,7 @@ class WithPmapAllAvailableDeviceTest(parameterized.TestCase):
     elif n_gpu > 1:
       self.n_devices, self.backend = n_gpu, 'gpu'
     else:
-      self.n_devices, self.backend = FLAGS.chex_n_cpu_devices, 'cpu'
+      self.n_devices, self.backend = FLAGS['chex_n_cpu_devices'].value, 'cpu'
 
   @variants.variants(with_pmap=True)
   @parameterized.parameters(*DEFAULT_PARAMS)
